import { __awaiter } from 'tslib';
import { ɵɵdefineInjectable, ɵɵinject, Injectable } from '@angular/core';
import { MtmCscApiBaseModel, MtmCscApi } from 'mtm-csc-api';
import { DcEasApi } from 'dc-eas-api';

class BaseFactory {
    constructor() {
        this._s = {};
        this._service = null;
    }
    setService(service) {
        this._service = service;
        return this;
    }
    getService() {
        return this._service;
    }
    getSession() {
        return this.getService().getUserSessions().getCurrent();
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var sessObj = yield this.getSession();
            return yield sessObj.newRequest();
        });
    }
}

class BaseModel extends MtmCscApiBaseModel {
    constructor() {
        super(...arguments);
        this._s = {};
        this._service = null;
        this._pushObj = null;
    }
    setService(service) {
        this._service = service;
        this.setAuto(service.getMtmAuto());
        return this;
    }
    getService() {
        return this._service;
    }
    getPush() {
        return this._pushObj;
    }
    setPush(value) {
        this._pushObj = value;
        return this;
    }
    push() {
        return this.getParent().push(this);
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var sessObj = yield this.getService().getUserSessions().getCurrent();
            return yield sessObj.newRequest();
        });
    }
}

class UserSession extends BaseModel {
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            reqObj.addReq("dpi-session-guid", this.getGuid());
            reqObj.addReq("authHash", this.getAuthHash());
            return reqObj;
        });
    }
    getAuthHash() {
        return this.getKey("authHash", true);
    }
    getCreated() {
        return this.getKey("created", true);
    }
}

class UserSessions extends BaseFactory {
    constructor() {
        super(...arguments);
        this._curObj = null;
    }
    newObj() {
        var rObj = new UserSession();
        rObj.setService(this.getService());
        return rObj;
    }
    getCurrent() {
        if (this._curObj === null) {
            this._curObj = this.getService().getMtmAuto().newPromise();
            this.addByEas(true).then((sessObj) => {
                this._curObj["resolve"](sessObj);
                console.log(sessObj);
            }, (errObj) => {
                var curObj = this._curObj;
                this._curObj = null;
                curObj["reject"](errObj);
            });
        }
        return this._curObj.promise;
    }
    addByEas(doThrow = false) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log("DPI authentication with EAS");
            var keyObj = yield this.getService().getEasApi().getEphemeralKeys().newReadIdKey();
            var reqObj = yield this.getService().newRequest();
            reqObj.setL1("UserSessions").setL2("Add").setL3("ByEas");
            yield keyObj.signPermissionRequest(reqObj);
            try {
                var apiObj = yield reqObj.getResponse(true);
                if (typeof this._s[apiObj.guid] === "undefined") {
                    this._s[apiObj.guid] = this.newObj();
                }
                this._s[apiObj.guid].set(apiObj);
                console.log("DPI Authenticated!");
                return this._s[apiObj.guid];
            }
            catch (errObj) {
                if (doThrow === true) {
                    throw errObj;
                }
                else {
                    return null;
                }
            }
        });
    }
}

class Device extends BaseModel {
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.addReq("device-guid", this.getGuid());
            return reqObj;
        });
    }
    getLabel() {
        return this.getKey("label", true);
    }
    getNamespace() {
        return this.getKey("namespace", true);
    }
    getNsId() {
        return this.getKey("nsId", true);
    }
    getCreated() {
        return this.getKey("created", true);
    }
}

class Devices extends BaseFactory {
    constructor() {
        super(...arguments);
        this._allRetrieved = false;
    }
    newObj() {
        var rObj = new Device();
        rObj.setService(this.getService());
        return rObj;
    }
    getAll() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._allRetrieved === false) {
                var reqObj = yield this.newRequest();
                reqObj.setL1("Devices").setL2("Get").setL3("All");
                var apiObjs = yield reqObj.getResponse(true);
                for (let apiObj of apiObjs) {
                    if (typeof this._s[apiObj.guid] === "undefined") {
                        this._s[apiObj.guid] = this.newObj();
                    }
                    this._s[apiObj.guid].set(apiObj);
                }
                this._allRetrieved = true;
            }
            return Object.values(this._s);
        });
    }
    getByGuid(guid) {
        return __awaiter(this, void 0, void 0, function* () {
            var devObj = null;
            var devObjs = Object.values(this._s);
            for (let x = 0; x < devObjs.length; ++x) {
                devObj = devObjs[x];
                if (devObj.getGuid() === guid) {
                    return devObj;
                }
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Devices").setL2("Get").setL3("ByGuid");
            reqObj.addReq("device-guid", guid);
            var apiObj = yield reqObj.getResponse(true);
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
            }
            this._s[apiObj.guid].set(apiObj);
            return this._s[apiObj.guid];
        });
    }
}

class DcDpiApi {
    constructor(_dcEasApi, _mtmCscApi) {
        this._dcEasApi = _dcEasApi;
        this._mtmCscApi = _mtmCscApi;
        this._procObj = null;
        this._usFact = null;
        this._devFact = null;
        this._wsSecret = null;
        this._wsHost = null;
        this._wsPort = null;
        this._wsSsl = true;
        this.initialize();
    }
    initialize() {
    }
    getEasApi() {
        return this._dcEasApi;
    }
    getMtmAuto() {
        return this._mtmCscApi.getAuto();
    }
    getUserSessions() {
        if (this._usFact === null) {
            this._usFact = new UserSessions();
            this._usFact.setService(this);
        }
        return this._usFact;
    }
    getDevices() {
        if (this._devFact === null) {
            this._devFact = new Devices();
            this._devFact.setService(this);
        }
        return this._devFact;
    }
    setWsConfig(host, port, secret, ssl = true) {
        this._wsSecret = secret;
        this._wsHost = host;
        this._wsPort = port;
        this._wsSsl = ssl;
        return this;
    }
    getProcObj() {
        if (this._procObj === null) {
            this._procObj = this.getMtmAuto().newPromise();
            console.log("Establishing DPI WSS connection");
            this._mtmCscApi.getProcesses().setWsConfig(this._wsSecret, this._wsHost, this._wsPort, this._wsSsl);
            this._mtmCscApi.getProcesses().getV1().initialize().then((procObj) => {
                this._procObj["data"] = procObj;
                this._procObj["resolve"](procObj);
                console.log("DPI WSS api enabled!");
            }, (errObj) => {
                var procObj = this._procObj;
                this._procObj = null;
                procObj["reject"](errObj);
                console.log("WSS api error:");
                console.log(errObj);
            });
        }
        return this._procObj.promise;
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var procObj = yield this.getProcObj();
            return yield procObj.newRequest();
        });
    }
}
DcDpiApi.ɵprov = ɵɵdefineInjectable({ factory: function DcDpiApi_Factory() { return new DcDpiApi(ɵɵinject(DcEasApi), ɵɵinject(MtmCscApi)); }, token: DcDpiApi, providedIn: "root" });
DcDpiApi.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
DcDpiApi.ctorParameters = () => [
    { type: DcEasApi },
    { type: MtmCscApi }
];

/*
 * Public API Surface of mtm-csc-api
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DcDpiApi };
//# sourceMappingURL=dc-dpi-api.js.map
